//
// Created by philipp on 22.03.18.
//

#include <pluginlib/class_list_macros.h>
#include <ros/ros.h>
#include "logging/Logging.h"

PLUGINLIB_EXPORT_CLASS(car::Logging, nodelet::Nodelet);

namespace car
{

constexpr std::size_t Logging::PORT;
constexpr std::size_t Logging::MAX_MESSAGE_SIZE;
constexpr std::size_t Logging::MAX_BUFFER_SIZE;

Logging::Logging()
    : loggingBuffer(MAX_BUFFER_SIZE)
      , loggingOStream(&loggingBuffer)
{}

void Logging::onInit()
{
    std::cout << ">> Logging [onInit] START\n";

    camData = nh_.subscribe("camData", 1, &Logging::camDataCallback, this);
    ccData = nh_.subscribe("ccData", 1, &Logging::ccDataCallback, this);
    environmentData = nh_.subscribe("environmentData", 1, &Logging::environmentDataCallback, this);
    laneData = nh_.subscribe("laneData", 1, &Logging::laneDataCallback, this);
    logEnabled = nh_.subscribe("logEnabled", 1, &Logging::logEnabledCallback, this);
    logString = nh_.subscribe("logString", 1000, &Logging::logStringCallback, this);
    rcEnabled = nh_.subscribe("rcEnabled", 1, &Logging::rcEnabledCallback, this);
    stmData = nh_.subscribe("stmData", 1, &Logging::stmDataCallback, this);
    ussData = nh_.subscribe("ussData", 1, &Logging::ussDataCallback, this);

    loggingServer = networking::service::Server<LoggingService>::create(net, PORT, MAX_MESSAGE_SIZE);
    loggingServer->advertiseService(
        [this](const auto & clientEndpoint, auto & requestMessage, auto & responseMessage)
        {
            std::lock_guard<std::mutex> lock{loggingMutex};
            responseMessage = networking::utils::stringFromStreambuf(
                loggingBuffer, std::min(loggingBuffer.size(), MAX_MESSAGE_SIZE));
        });

    std::cout << ">> Logging [onInit] END\n";
}

void Logging::ccDataCallback(const ccDataMsg::ConstPtr & inMsg)
{

}

void Logging::camDataCallback(const camDataMsg::ConstPtr & inMsg)
{

}

void Logging::environmentDataCallback(const environmentDataMsg::ConstPtr & inMsg)
{
    std::lock_guard<std::mutex> lock{loggingMutex};
    loggingOStream << ">> environmentData [distance] " << inMsg->distance << "\n";
}

void Logging::laneDataCallback(const laneDataMsg::ConstPtr & inMsg)
{

}

void Logging::logEnabledCallback(const logEnabledMsg::ConstPtr & inMsg)
{

}

void Logging::logStringCallback(const logStringMsg::ConstPtr & inMsg)
{
    std::lock_guard<std::mutex> lock{loggingMutex};
    loggingOStream << inMsg->logMsg;
}

void Logging::rcEnabledCallback(const rcEnabledMsg::ConstPtr & inMsg)
{

}

void Logging::stmDataCallback(const stmDataMsg::ConstPtr & inMsg)
{

}

void Logging::ussDataCallback(const ussDataMsg::ConstPtr & inMsg)
{
    std::lock_guard<std::mutex> lock{loggingMutex};
    loggingOStream << ">> ussData [distance] " << inMsg->distance << "\n";
}

}